/*
 * @(#)TicketSeller.java  1.0  5 October 2001
 *
 * Copyright 2001
 * College of Computer Science
 * Northeastern University
 * Boston, MA  02115
 *
 * This software may be used for educational purposes as long as
 * this copyright notice is retained intact at the top of all files.
 *
 * Should this software be modified, the words "Modified from 
 * Original" must be included as a comment below this notice.
 *
 * All publication rights are retained.  This software or its 
 * documentation may not be published in any media either in whole
 * or in part without explicit permission.
 *
 * Contact information:
 *   Richard Rasala    rasala@ccs.neu.edu
 *   Viera Proulx      vkp@ccs.neu.edu
 *   Jeff Raab         jmr@ccs.neu.edu
 * 
 * Telephone:          617-373-2462
 *
 * This software was created with support from Northeastern 
 * University and from NSF grant DUE-9950829.
 */

import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A class that simulates sales of tickets to one theater show
 * with general admission and three categories of prices
 *
 * @author Viera K. Proulx 
 * @author Richard Rasala
 * @version 27 August 2001
 */
public class TicketSeller extends DisplayPanel
  implements JPTConstants {
  
  ////////////////////
  // View Constants // 
  ////////////////////        
  
  // constants: used to set and reset the annotation fields
  private static final String NO_TICKETS   = "0";                
  private static final String NO_SALES     = "$0";                
  private static final String DEFAULT_SHOW = "Show not specified";
  
  // Input text field error prompt, error title, width, and default
  private static final String INT_ERROR = "Number of tickets must be an integer:";
  private static final String INT_ERROR_TITLE = "Ticket Error";
  private static final int TFV_WIDTH = 60;
  
  // Main panel width
  private static final int MAIN_PANEL_WIDTH = 400;
  
  
  /////////////////
  // Member Data //
  /////////////////
  
  // current show object
  private Show currentShow = null;
  
  // three Show objects that will record their prices, and ticket sales
  private Show tarzanShow = new Show("Tarzan", 200, 20, 12, 8);
  private Show kingKongShow = new Show("King Kong", 250, 15, 10, 5);
  private Show casablancaShow = new Show("Casablanca", 300, 10, 5, 5);
  
  //////////////////
  // View Section //
  //////////////////
  
  ////////////////
  // Show title //
  ////////////////
  
  private Annotation showTitleField =
    new Annotation(DEFAULT_SHOW);
  
  private Annotation showTitleFieldColor =
    new Annotation(DEFAULT_SHOW);
  
  /////////////////////////////////////
  // Sales statistics display fields //
  /////////////////////////////////////
  
  // adultSalesField will be used to show the number of adult tickets sold  
  private Annotation adultSalesField =
    new Annotation(NO_TICKETS);
  
  // studentSalesField will be used to show the number of student tickets sold  
  private Annotation studentSalesField =
    new Annotation(NO_TICKETS);
  
  // childSalesField will be used to show the number of child tickets sold  
  private Annotation childSalesField =
    new Annotation(NO_TICKETS);
  
  // totalSalesField will be used to show the number of tickets sold  
  private Annotation totalSalesField =
    new Annotation(NO_TICKETS);
  
  // adultIncomeField will be used to show the income for adult tickets sold  
  private Annotation adultIncomeField =
    new Annotation(NO_SALES);
  
  // studentIncomeField will be used to show the income for student tickets sold  
  private Annotation studentIncomeField =
    new Annotation(NO_SALES);
  
  // childIncomeField will be used to show the income for child tickets sold  
  private Annotation childIncomeField =
    new Annotation(NO_SALES);
  
  // totalIncomeField will be used to show the total income for tickets sold  
  private Annotation totalIncomeField =
    new Annotation(NO_SALES);
  
  /////////////////////////////////
  // Sales window display fields //
  /////////////////////////////////
  
  // availableTicketsField will be used to show the available seats  
  private Annotation availableTicketsField =
    new Annotation(NO_TICKETS);
  
  // number of adult tickets requested
  private TextFieldView adultOrderTFV =
    new TextFieldView(
                      NO_TICKETS,                 // initial value displayed in the TFV
                      INT_ERROR,       // prompt for correction
                      INT_ERROR_TITLE,          // title for the error dialog box
                      TFV_WIDTH);                 // preferred width for the TFV
  
  // number of student tickets requested
  private TextFieldView studentOrderTFV =
    new TextFieldView(
                      NO_TICKETS,                 // initial value displayed in the TFV
                      INT_ERROR,       // prompt for correction
                      INT_ERROR_TITLE,          // title for the error dialog box
                      TFV_WIDTH);                 // preferred width for the TFV
  
  // number of child tickets requested
  private TextFieldView childOrderTFV =
    new TextFieldView(
                      NO_TICKETS,                 // initial value displayed in the TFV
                      INT_ERROR,       // prompt for correction
                      INT_ERROR_TITLE,          // title for the error dialog box
                      TFV_WIDTH);                 // preferred width for the TFV
  
  // total of tickets requested 
  private Annotation totalOrderField =
    new Annotation(NO_TICKETS);
  
  // price of an adult ticket 
  private Annotation adultPriceField =
    new Annotation(NO_SALES);
  
  // price of a student ticket 
  private Annotation studentPriceField =
    new Annotation(NO_SALES);
  
  // price of a child ticket 
  private Annotation childPriceField =
    new Annotation(NO_SALES);
  
  // price of tickets requested 
  private Annotation totalPriceField =
    new Annotation(NO_SALES);
  
  
  ////////////////////
  // Action Section //
  ////////////////////
  
  // the actions for the show settings actions panel
  
  // action to reset all sales statistics for the current show
  private Action reset = 
    new SimpleAction("Reset Sales") {
    public void perform(){ reset(); }
  };
  
  // action make Tarzan the current show
  private Action tarzan = 
    new SimpleAction("Tarzan") {
    public void perform(){ tarzan(); }
  };
  
  // action make King Kong the current show
  private Action kingKong = 
    new SimpleAction("King Kong") {
    public void perform(){ kingKong(); }
  };
  
  // action make Casablanca the current show
  private Action casablanca = 
    new SimpleAction("Casablanca") {
    public void perform(){ casablanca(); }
  };
  
  // action to clear the ticket order fields
  private Action clear = 
    new SimpleAction("Clear") {
    public void perform(){ clear(); }
  };
  
  // action to compute the total number of tickets and price
  private Action total = 
    new SimpleAction("Total") {
    public void perform(){ total(); }
  };
  
  // action to buy the total number of tickets
  private Action buy = 
    new SimpleAction("Buy") {
    public void perform(){ buy(); }
  };
  
  // list of actions to be included in the reset actions panel    
  private Action[] resetActionList = { tarzan, kingKong, casablanca, reset };
  
  // reset actions panel
  private ActionsPanel resetActions = new ActionsPanel(resetActionList);
  
  
  // list of actions to be included in the order actions panel
  private Action[] orderActionList = { clear, total, buy };
  
  // order actions panel
  private ActionsPanel orderActions = new ActionsPanel(orderActionList);
  
  
  //////////////////////////////////////////////
  // GUI Section: Combining views into panels //
  //////////////////////////////////////////////
  
  // Panel to display total tickets sold and ticket income
  private TablePanel ticketSalesPanel = 
    new TablePanel(
                   new Object[][] {
    { "Show:",     showTitleField,    "" },
    { "",          "Tickets",         "Income"    },
    { "Adult:",    adultSalesField,   adultIncomeField   },
    { "Student:",  studentSalesField, studentIncomeField },
    { "Child:",    childSalesField,   childIncomeField   },
    { "Total:",    totalSalesField,   totalIncomeField   }},
                   10, 5);
  
  // Panel to display ticket order information and request
  private TablePanel ticketOrderPanel = 
    new TablePanel(
                   new Object[][] {
    {  showTitleFieldColor,  "Tickets Available",    availableTicketsField },
    { "",                    "Tickets",             "Prices"   },
    { "Number of Adult:",     adultOrderTFV,         adultPriceField   },
    { "Number of Student:",   studentOrderTFV,       studentPriceField },
    { "Number of Child:",     childOrderTFV,         childPriceField   },
    { "Total:",               totalOrderField,       totalPriceField   }},
                   10, 5);
  
  
  // wrappers that label each panel
  private Display ticketSalesDisplay = 
    new Display(ticketSalesPanel, null, "Sales Statistics");
  
  private Display ticketOrderDisplay = 
    new Display(ticketOrderPanel, null, "Ticket Order and Prices");
  
  
  // the enclosing panel 
  // that contains sales statistics, box office information and actions
  private TablePanel mainPanel = 
    new TablePanel(
                   new Object[] {
    ticketSalesDisplay,
      resetActions,
      ticketOrderDisplay,
      orderActions },
                   VERTICAL, 0, 10, CENTER);
  
  // minimum row heights
  private int minimumHeights = 0;
  
  //////////////////        
  // Main Program //
  //////////////////
  
  // create the main ticket seller display panel
  // call its constructor to install all the inner panels
  public static void main(String[] args) {
    
    JPTFrame.createQuickJPTFrame(
                                 "Ticket Seller",            // title for the main application GUI
                                 new TicketSeller());        // call to the TicketSeller constructor
  }
  
  
  /////////////////
  // Constructor //
  /////////////////
  
  // constructor for the ticket seller application
  // create and initialize the show to sell
  // arrange the displays neatly
  // 
  public TicketSeller() {
    
    // make the initial show to be Tarzan
    tarzan();
    
    // display the show title and available tickets in red 
    // in the ticket order display
    showTitleFieldColor.setForeground(Color.red);
    availableTicketsField.setForeground(Color.red);
    
    //////////////////        
    // View Section //    
    //////////////////        
    
    // set the minimum height of the fields in the ticket order display
    minimumHeights = adultOrderTFV.getPreferredSize().height;
    
    adjustTableSettings(ticketOrderPanel);
    
    mainPanel.setMinimumColumnWidth(0, MAIN_PANEL_WIDTH);
    
    // add the main panel to the main display
    add(mainPanel);
  }
  
  
  ////////////////////////
  // Action Definitions //
  ////////////////////////  
  
  // function to reset the current show sales statistics to the initial state
  // and clear the display in the ticket order window
  public void reset() {
    
    // reset the current show statistics and show the new information
    currentShow.reset();
    updateSalesView();
    
    //clear the box office request views and totals of tickets and price
    clear();
  }
  
  // function to reset the current show to be Tarzan
  public void tarzan() {
    
    // set the current show to be Tarzan
    currentShow = tarzanShow;
    
    // show current title and sales statistics    
    updateSalesView();
    
    // show prices for this show
    updateShowView();
    
    //clear the box office request views and totals of tickets and price
    clear();
  }
  
  // function to reset the current show to be King Kong
  public void kingKong() {
    
    // set the current show to be King Kong
    currentShow = kingKongShow;
    
    // show current title and sales statistics    
    updateSalesView();
    
    // show prices for this show
    updateShowView();
    
    //clear the box office request views and totals of tickets and price
    clear();
  }
  
  // function to reset the current show to be Casablanca
  public void casablanca() {
    
    setCurrentShow(casablancaShow);
    /*
     * 
     // set the current show to be Casablanca
     currentShow = casablancaShow;
     
     // show current title and sales statistics    
     updateSalesView();
     
     // show prices for this show
     updateShowView();
     
     //clear the box office request views and totals of tickets and price
     clear();
     */
  }
  
//??//??//??//??//??//?? solution to bonus question AF-7 //??//??//??//??//??//
  // function to reset the current show to be the specified show    
  private void setCurrentShow(Show show){
    
    // set the current show to be the specified
    currentShow = show;
    
    // show current title and sales statistics    
    updateSalesView();
    
    // show prices for this show
    updateShowView();
    
    //clear the box office request views and totals of tickets and price
    clear();
  }
  
  // function to clear the display in the ticket order window
  public void clear(){
    
    adultOrderTFV.setViewState(NO_TICKETS);
    
//??//??//??//??//??//?? omiteed in student code AF-2 //??//??//??//??//??//
    studentOrderTFV.setViewState(NO_TICKETS);
    
    childOrderTFV.setViewState(NO_TICKETS);
    
    totalOrderField.setText(NO_TICKETS);
    totalPriceField.setText(NO_SALES);
  }
  
  
  // function to compute and display the total price of tickets
  // displays sold out message if the ticket order exceeds availability
  public void total() {
    
    // get the ticket order information
    int a = adultOrderTFV.demandInt();
    int s = studentOrderTFV.demandInt();
    int c = childOrderTFV.demandInt();
    
    if ((a < 0) || (s < 0) || (c < 0)){
      totalPriceField.setText("Error");
      return;
    }
    
    // compute the total number of tickets requested and display the result
    int tickets = a + s + c;
    totalOrderField.setText(string(tickets));
    
    if (currentShow.available(tickets)) {
      int price = currentShow.price(a, s, c);     
      totalPriceField.setText(dollar(price));
    }
    else
      totalPriceField.setText("Sold out");
    
  }
  
  // function to process the order
  // if desired tickets are available, adjust show statistics
  // and display all relevant information
  public void buy() {
    
    // get the ticket order information
    int a = adultOrderTFV.demandInt();
    int s = studentOrderTFV.demandInt();
    int c = childOrderTFV.demandInt();
    
    // compute the total number of tickets requested and display the result
    int tickets = a + s + c;
    totalOrderField.setText(string(tickets));
    
    // if tickets are available, sell the tickets and show the order price
    if (currentShow.available(tickets)){
      int price = currentShow.sell(a, s, c);     
      totalPriceField.setText(dollar(price));
      updateSalesView();
    }
    // show message if the total number of tickets requested is not available
    else
      totalPriceField.setText("Sold out");
  }
  
  
  //////////////////////////////
  // Helper/Utility functions //
  //////////////////////////////
  
  
  // update the displays of unchanging information about the current show: 
  // show title and prices
  private void updateShowView(){
    
    // display the show title in sales statistics display
    showTitleField.setText(currentShow.getShowName());
    
    // display the show title in ticket order display
    showTitleFieldColor.setText(currentShow.getShowName());
    
    // show prices
    adultPriceField.setText  ( dollar(currentShow.getAdultPrice()   ) );
    
//??//??//??//??//?? update the student price annotation HF-1 //??//??//??//
    studentPriceField.setText( dollar(currentShow.getStudentPrice() ) );
    
    childPriceField.setText  ( dollar(currentShow.getChildPrice()   ) );
  }
  
  // update the displays related to this show's statistics,
  // including the ticket availability
  private void updateSalesView(){
    
    // update ticket availability information
    availableTicketsField.setText(string(currentShow.getAvailability()));     
    
    // update number of adult tickets sold and income information
    int adult = currentShow.getAdultsSold();
    int adultPrice = currentShow.getAdultPrice();
    int adultIncome = adult * adultPrice;
    
    adultSalesField.setText(string(adult));
    adultIncomeField.setText(dollar(adultIncome));
    
//??//??//??//??//??//?? Question HF-3 //??//??//??//??//??//??//??//??//??//
    // update number of student tickets sold and income information
    int student = currentShow.getStudentsSold();
    int studentPrice = currentShow.getStudentPrice();
    int studentIncome = student * studentPrice;
    
    studentSalesField.setText(string(student));
    studentIncomeField.setText(dollar(studentIncome));
    
    // update number of children tickets sold and income information
    int child = currentShow.getChildrenSold();
    int childPrice = currentShow.getChildPrice();
    int childIncome = child * childPrice;
    
    childSalesField.setText(string(child));
    childIncomeField.setText(dollar(childIncome));
    
    // update the display of total number of tickets sold and total income
    totalSalesField.setText(string(adult + student + child));
    
//??//??//??//??//??//?? Question HF-4 //??//??//??//??//??//??//??//??//??//
    totalIncomeField.setText(dollar(adultIncome + 
                                    studentIncome + 
                                    childIncome));     
  }
  
  
  // helper function that adjusts the heights ahd widths of the display tables
  private void adjustTableSettings(TablePanel panel) {
    panel.setMinimumColumnWidth(1, TFV_WIDTH);
    panel.setMinimumColumnWidth(2, TFV_WIDTH);
    
    panel.setColumnAlignment(1, SOUTH_EAST);
    panel.setColumnAlignment(2, SOUTH_EAST);
    
    panel.setRowAlignment(0, SOUTH);
    
    panel.setAllMinimumRowHeights(minimumHeights);
  }
  
  // helper function to convert an int to String for the displays
  private static String string(int count) {
    return "" + count;
  }
  // helper function to convert an int to $value to display prices
  private static String dollar(int price) {
    return "$" + price;
  }
}